package com.winhc.repal.model.vo;

import com.fasterxml.jackson.annotation.JsonFormat;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import lombok.Getter;
import lombok.Setter;
import lombok.ToString;
import org.springframework.format.annotation.DateTimeFormat;

import java.time.LocalDate;
import java.time.LocalDateTime;
import java.util.List;

/**
 * @Description: RepalBillDetailVO
 * @Author: xda
 * @Date: 2022/4/13 19:40
 */
@Data
@ApiModel("账款详情")
public class RepalBillDetailVO {

    @ApiModelProperty(value = "所属组织id")
    private Long repalGroupId;

    @ApiModelProperty(value = "账款id")
    private Long repalBillId;

    @ApiModelProperty(value = "账本id")
    private Long repalBookId;

    @ApiModelProperty(value = "客户name")
    private String customerName;

    @ApiModelProperty(value = "客户id")
    private Long customerId;

    @ApiModelProperty(value = "公司companyId")
    private String companyId;

    @ApiModelProperty(value = "客户性质 1企业 2个人 3政府事业单位")
    private Integer custProperty;

    @ApiModelProperty("失信限高标签list")
    private List<String> labelList;

    @ApiModelProperty(value = "负责人name，、分隔")
    private String responsiblePersonNames;

    @ApiModelProperty("到期日期,字符串,格式yyyy-MM-dd")
    @JsonFormat(pattern = "yyyy-MM-dd")
    private LocalDate endDate;

    @ApiModelProperty(value = "账款总金额")
    private String totalMoneyStr;

    @ApiModelProperty(value = "待收金额/待付金额")
    private String receivableStr;

    @ApiModelProperty(value = "账款已收金额")
    private String receiptStr;

    @ApiModelProperty(value = "分数趋势，1上升，2下降。0持平")
    private Integer diagnosisScoreTrend;

    @ApiModelProperty(value = "评估等级")
    private String diagnosisGrade;

    @ApiModelProperty(value = "本次评估等级描述")
    private String diagnosisGradeDesc;

    @ApiModelProperty(value = "本次评估分数")
    private String diagnosisScore;

    @ApiModelProperty(value = "账单状态,1进行中，2已逾期，3已收回，4已终止")
    private Integer repalBillStatus;

    @ApiModelProperty(value = "账单刷新状态。1未刷新，2刷新中，3已刷新，0刷新失败")
    private Integer refreshStatus;

    @ApiModelProperty(value = "账款阶段,1未起诉，2已起诉，3已申请执行，4终结本次执行")
    private Integer repalBillStage;

    @ApiModelProperty(value = "账款备注")
    private String billRemark;

    @ApiModelProperty(value = "风险数量)")
    private Integer riskNum;

    @ApiModelProperty("财产线索)")
    private Integer assetsNum;

    @ApiModelProperty("账款分析4个纬度")
    private List<String> analysisList;

    @ApiModelProperty(value = "合同name")
    private String contractName;

    @ApiModelProperty(value = "合同到期日期")
    @JsonFormat(pattern = "yyyy-MM-dd")
    private LocalDate contractEndDate;

    @ApiModelProperty("合同列表")
    private List<BillContract> contractList;

    @ApiModelProperty(value = "逾期天数/离收款日")
    private Integer overdueDay;

    @ApiModelProperty(value = "收付款记录list")
    private List<BillMoneyChange> changeList;

    @ApiModelProperty(value = "负责人列表")
    private List<ResponsiblePersonNameVO> responsiblePersonList;


    @Getter
    @Setter
    @ToString
    @ApiModel("金额变动记录")
    public static class BillMoneyChange {

        @ApiModelProperty("金额string")
        private String changeAmt;

        @ApiModelProperty("操作时间,string, yyyy-MM-dd HH:mm:ss")
        @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss")
        @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
        private LocalDateTime changeTime;

        @ApiModelProperty("操作人name")
        private String name;

        @ApiModelProperty("变更记录id")
        private Long repalChangeRecordId;

        @ApiModelProperty("操作类型文案")
        private String operTypeDesc;



    }

    @Getter
    @Setter
    @ToString
    @ApiModel("合同")
    public static class BillContract {

        @ApiModelProperty("文件url")
        private String contractUrl;

        @ApiModelProperty("文件name")
        private String contractName;
    }

}
