package com.winhc.repal.service.impl;

import cn.hutool.core.collection.CollUtil;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.winhc.common.enums.CodeMsg;
import com.winhc.common.exception.CommonException;
import com.winhc.repal.entity.RepalBill;
import com.winhc.repal.entity.RepalChangeRecord;
import com.winhc.repal.entity.RepalGroupMember;
import com.winhc.repal.enums.DeletedStatusEnum;
import com.winhc.repal.enums.RepalBillStatusEnum;
import com.winhc.repal.model.dto.ReturnedEditDTO;
import com.winhc.repal.repository.RepalChangeRecordMapper;
import com.winhc.repal.service.RepalBillService;
import com.winhc.repal.service.RepalChangeRecordService;
import com.winhc.repal.service.RepalGroupMemberService;
import com.winhc.repal.util.UserContextUtil;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.util.List;
import java.util.Objects;

/**
 * @description RepalChangeRecord 接口实现类
 * @author Generator
 * @date 2022-04-08
 */
@Service
public class RepalChangeRecordServiceImpl extends ServiceImpl<RepalChangeRecordMapper, RepalChangeRecord> implements RepalChangeRecordService {

    @Autowired
    private RepalChangeRecordMapper repalChangeRecordMapper;

    @Autowired
    private RepalBillService repalBillService;

    @Autowired
    private RepalGroupMemberService repalGroupMemberService;

    @Override
    public List<RepalChangeRecord> getHomeRecord(Long userId, Long repalBookId, LocalDate start, LocalDate end) {
        return repalChangeRecordMapper.getHomeRecord(userId, repalBookId, start, end);
    }


    @Override
    @Transactional(rollbackFor = Exception.class)
    public Boolean updateReturned(ReturnedEditDTO dto) {
        if (dto.getMoney().compareTo(BigDecimal.ZERO) <= 0) {
            throw new CommonException(CodeMsg.FAILED, "金额必须大于0");
        }
        RepalChangeRecord repalChangeRecord = this.getById(dto.getRepalChangeRecordId());
        if (Objects.isNull(repalChangeRecord)) {
            throw new CommonException(CodeMsg.FAILED, "未查到对应的账款");
        }
        if (repalChangeRecord.getTransMoney().compareTo(dto.getMoney()) == 0) {
            return true;
        }
        RepalBill repalBill = repalBillService.getById(repalChangeRecord.getRepalBillId());
        if (dto.getMoney().compareTo(repalChangeRecord.getTransMoney()) > 0) {
            if (dto.getMoney().subtract(repalChangeRecord.getTransMoney()).compareTo(repalBill.getReceivable()) > 0) {
                throw new CommonException(CodeMsg.FAILED, "付款金额不能大于应付金额");
            }
        }
        List<RepalChangeRecord> recordList = this.list(Wrappers.lambdaQuery(RepalChangeRecord.class)
                .eq(RepalChangeRecord::getRepalBillId, repalBill.getId())
                .eq(RepalChangeRecord::getDeleted, DeletedStatusEnum.NORMAL.getCode())
                .ge(RepalChangeRecord::getId, repalChangeRecord.getId()));
        if (CollUtil.isEmpty(recordList)) {
            throw new CommonException(CodeMsg.FAILED, "未查到对应的账款");
        }
        // 单独修改一条，后续的重新计算
        RepalGroupMember member = repalGroupMemberService.getMemberByUserId(UserContextUtil.getUser().getUserId());
        for (int i = 0; i < recordList.size(); i++) {
            if (i == 0) {
                recordList.get(i).setOperUserName(member.getMemberUserName());
                recordList.get(i).setOperUserId(member.getUserId());
                recordList.get(i).setRecordDate(dto.getRecordDate());
                recordList.get(i).setTransMoney(dto.getMoney());
            } else {
                recordList.get(i).setPreMoney(recordList.get(i - 1).getAfterMoney());
            }
            recordList.get(i).setAfterMoney(recordList.get(i).getPreMoney().subtract(recordList.get(i).getTransMoney()));
        }
        this.saveOrUpdateBatch(recordList);
        // 修改账款状态或者金额
        if (dto.getMoney().compareTo(repalChangeRecord.getTransMoney()) > 0) {
            // 应收金额调小
            BigDecimal changeMoney = dto.getMoney().subtract(repalChangeRecord.getTransMoney());
            repalBill.setReceivable(repalBill.getReceivable().subtract(changeMoney));
            repalBill.setReceipt(repalBill.getReceipt().add(changeMoney));
            if (BigDecimal.ZERO.compareTo(repalBill.getReceivable()) == 0) {
                repalBill.setRepalBillStatus(RepalBillStatusEnum.SUCCESS.getCode());
            }
        } else {
            // 应收金额调大
            BigDecimal changeMoney = repalChangeRecord.getTransMoney().subtract(dto.getMoney());
            repalBill.setReceivable(repalBill.getReceivable().add(changeMoney));
            repalBill.setReceipt(repalBill.getReceipt().subtract(changeMoney));
            if (RepalBillStatusEnum.SUCCESS.getCode().equals(repalBill.getRepalBillStatus())) {
                if (LocalDate.now().isBefore(repalBill.getEndDate())) {
                    repalBill.setRepalBillStatus(RepalBillStatusEnum.OVERDUE.getCode());
                }
                if (LocalDate.now().isAfter(repalBill.getEndDate())) {
                    repalBill.setRepalBillStatus(RepalBillStatusEnum.PROGRESSING.getCode());
                }
            }
        }
        repalBillService.update(Wrappers.lambdaUpdate(RepalBill.class)
                .eq(RepalBill::getId, repalBill.getId())
                .set(RepalBill::getRepalBillStatus, repalBill.getRepalBillStatus())
                .set(RepalBill::getReceivable, repalBill.getReceivable())
                .set(RepalBill::getReceipt, repalBill.getReceipt()));
        return true;
    }



    @Override
    public Boolean delReturnedByBillId(Long repalBillId) {
        this.update(Wrappers.lambdaUpdate(RepalChangeRecord.class)
                .eq(RepalChangeRecord::getRepalBillId, repalBillId)
                .set(RepalChangeRecord::getDeleted, DeletedStatusEnum.DELETE.getCode()));
        return true;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Boolean delReturnedByBookId(Long repalBookId) {
        List<Long> billIdList = repalBillService.getBillIdListByBookId(repalBookId, null);
        if (CollUtil.isEmpty(billIdList)) {
            return true;
        }
        for (Long aLong : billIdList) {
            this.delReturnedByBillId(aLong);
        }
        return true;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Boolean delReturnedById(Long repalChangeRecordId) {
        RepalChangeRecord repalChangeRecord = this.getById(repalChangeRecordId);
        RepalBill repalBill = repalBillService.getById(repalChangeRecord.getRepalBillId());
        this.calculateMoney(repalChangeRecordId, repalBill.getId());
        repalBill.setRepalBillStatus(LocalDate.now().isBefore(repalBill.getEndDate()) ? RepalBillStatusEnum.OVERDUE.getCode() : RepalBillStatusEnum.PROGRESSING.getCode());
        repalBill.setReceipt(repalBill.getReceipt().subtract(repalChangeRecord.getTransMoney()));
        repalBill.setReceivable(repalBill.getReceivable().add(repalChangeRecord.getTransMoney()));
        // 删除后一定是进行中或者已逾期
        repalBillService.update(Wrappers.lambdaUpdate(RepalBill.class)
                .eq(RepalBill::getId, repalBill.getId())
                .set(RepalBill::getRepalBillStatus, repalBill.getRepalBillStatus())
                .set(RepalBill::getReceivable, repalBill.getReceivable())
                .set(RepalBill::getReceipt, repalBill.getReceipt()));
        return true;
    }

    @Override
    public Boolean calculateMoney(Long repalChangeRecordId, Long legalCloudId) {
        List<RepalChangeRecord> recordList = this.list(Wrappers.lambdaQuery(RepalChangeRecord.class)
                .eq(RepalChangeRecord::getRepalBillId, legalCloudId)
                .eq(RepalChangeRecord::getDeleted, DeletedStatusEnum.NORMAL.getCode())
                .ge(RepalChangeRecord::getId, repalChangeRecordId));
        if (CollUtil.isEmpty(recordList)) {
            return true;
        }
        RepalGroupMember member = repalGroupMemberService.getMemberByUserId(UserContextUtil.getUser().getUserId());
        // 重新计算金额
        for (int i = 0; i < recordList.size(); i++) {
            if (i == 0) {
                recordList.get(i).setOperUserName(member.getMemberUserName());
                recordList.get(i).setOperUserId(member.getUserId());
                recordList.get(i).setDeleted(DeletedStatusEnum.DELETE.getCode());
            } else if (i == 1) {
                recordList.get(i).setPreMoney(recordList.get(0).getPreMoney());
                recordList.get(i).setAfterMoney(recordList.get(i).getPreMoney().subtract(recordList.get(i).getTransMoney()));
            } else {
                recordList.get(i).setPreMoney(recordList.get(i - 1).getAfterMoney());
                recordList.get(i).setAfterMoney(recordList.get(i).getPreMoney().subtract(recordList.get(i).getTransMoney()));
            }
        }
        this.saveOrUpdateBatch(recordList);
        return true;
    }
}
